-- Copyright (c) 2007 Frank Buss (fb@frank-buss.de)
-- See license.txt for license

library IEEE;
use IEEE.STD_LOGIC_1164.ALL; 
use IEEE.NUMERIC_STD.ALL; 
use work.ALL;

entity spartan3e_test is
    port(
        CLK_50MHZ : in std_logic;
        led : out std_logic_vector(7 downto 0);
        btn_east : in std_logic;
        btn_north : in std_logic;
        btn_south : in std_logic;
        btn_west : in std_logic
    );
end entity spartan3e_test;

architecture rtl of spartan3e_test is

component ram IS
    port (
    addr: IN std_logic_VECTOR(10 downto 0);
    clk: IN std_logic;
    din: IN std_logic_VECTOR(15 downto 0);
    dout: OUT std_logic_VECTOR(15 downto 0);
    we: IN std_logic);
end component;

    constant word_size : natural := 16;

    -- CPU RAM signals
    signal address  : unsigned(word_size - 1 downto 0);
    signal data     : unsigned(word_size - 1 downto 0);
    signal q        : std_logic_vector(word_size - 1 downto 0);
    signal wren     : std_logic := '0';

    -- test output port
    signal port0    : std_logic_vector(7 downto 0);

    -- reads RAM or input ports
    signal buffered_q : unsigned(word_size - 1 downto 0);

begin

    -- CPU RAM
    
  instance_ram: ram 
    port map (
        addr    => std_logic_vector(address(10 downto 0)),
        clk     => CLK_50MHZ,
        din     => std_logic_vector(data),
        we      => wren,
        dout    => q
    );

    instance_forth_core: entity forth_core
    port map (
        clock       => CLK_50MHZ,
        reset       => btn_south,
        address     => address,
        wren        => wren,
        data_out    => data,
        q           => buffered_q
    );

    process(CLK_50MHZ, btn_south, address, wren, data)
    begin
        if btn_south = '1' then
            port0 <= (others => '0');
        elsif rising_edge(CLK_50MHZ) then
            if address = x"ffff" and wren = '1' then
                port0 <= std_logic_vector(data(7 downto 0));
            end if;
        end if;
    end process;

    buffered_q <=
      to_unsigned(0, 13) & btn_west & btn_north & btn_east when address = x"fffe"
      else unsigned(q);
    led <= port0;

end architecture rtl;
