-- Copyright (c) 2007 Frank Buss (fb@frank-buss.de)
-- See license.txt for license

library IEEE;
use IEEE.STD_LOGIC_1164.ALL; 
use IEEE.NUMERIC_STD.ALL; 
use work.ALL;

entity t_rex_test is
    port(
		clock_50mhz : in std_logic;
		led : out std_logic_vector(7 downto 0);
		button : in std_logic_vector(3 downto 0);
		dip_switch : in std_logic_vector(3 downto 0);
		neg_reset : in std_logic
	);
end entity t_rex_test;

architecture rtl of t_rex_test is

	constant word_size : natural := 16;

	-- CPU RAM signals
    signal address  : std_logic_vector(word_size - 1 downto 0);
    signal data     : std_logic_vector(word_size - 1 downto 0);
    signal q        : std_logic_vector(word_size - 1 downto 0);
    signal wren     : std_logic := '0';

	-- test output port
    signal port0    : std_logic_vector(word_size - 1 downto 0);

	-- reads RAM or input ports
    signal buffered_q : std_logic_vector(word_size - 1 downto 0);

begin

	-- CPU RAM
    instance_ram: entity ram 
    port map (
		address		=> address(10 downto 0),
		clock		=> clock_50mhz,
		data		=> data,
		wren		=> wren,
		q			=> q
	);

    instance_forth_core: forth_core
    port map (
		clock				=> clock_50mhz,
		reset				=> not(neg_reset),
		unsigned(address) 	=> address,
		wren				=> wren,
		unsigned(data_out)	=> data,
		q					=> unsigned(buffered_q)
	);

    process(clock_50mhz, neg_reset, address, wren, data)
    begin
        if neg_reset = '0' then
			port0 <= (others => '0');
        elsif rising_edge(clock_50mhz) then
			if address = std_logic_vector(to_unsigned(-1, word_size)) and wren = '1' then
				port0 <= data;
			end if;
        end if;
    end process;

	buffered_q <= x"00" & dip_switch & (button xor x"f") when address = std_logic_vector(to_unsigned(-2, word_size)) else q;
	led <= port0(7 downto 0);

end architecture rtl;
